@@ -108,7 +108,8 @@ typedef struct inputctx {
   // been taken, both become NULL.
   struct initial_responses* initdata;
   struct initial_responses* initdata_complete;
-  bool failed;          // error initializing input automaton, abort
+  int kittykbd;        // kitty keyboard protocol support level
+  bool failed;         // error initializing input automaton, abort
 } inputctx;
 
 static inline void
@@ -495,6 +496,13 @@ load_ncinput(inputctx* ictx, ncinput *tni){
       }
     }
   }
+  // if the kitty keyboard protocol is in use, any input without an explicit
+  // evtype can be safely considered a PRESS.
+  if(ictx->kittykbd){
+    if(tni->evtype == NCTYPE_UNKNOWN){
+      tni->evtype = NCTYPE_PRESS;
+    }
+  }
   if(tni->modifiers == NCKEY_MOD_CTRL){ // exclude all other modifiers
     if(ictx->linesigs){
       if(tni->id == 'C'){
@@ -1068,7 +1076,8 @@ kitty_keyboard_cb(inputctx* ictx){
   if(ictx->initdata){
     ictx->initdata->kbdlevel = level;
   }
-  loginfo("kitty keyboard protocol level %u", level);
+  loginfo("kitty keyboard level %u (was %u)", level, ictx->kittykbd);
+  ictx->kittykbd = level;
   return 2;
 }
 
@@ -1300,7 +1309,7 @@ da2_cb(inputctx* ictx){
 
 // weird form of Ternary Device Attributes used only by WezTerm
 static int
-da3_cb(inputctx* ictx){
+wezterm_tda_cb(inputctx* ictx){
   if(ictx->initdata){
     loginfo("read ternary device attributes");
   }
@@ -1666,6 +1675,8 @@ tda_cb(inputctx* ictx){
       ictx->initdata->qterm = TERMINAL_TERMINOLOGY;
     }else if(strcmp(str, "464F4F54") == 0){ // "FOOT"
       ictx->initdata->qterm = TERMINAL_FOOT;
+    }else if(strcmp(str, "7E4B4445") == 0){
+      ictx->initdata->qterm = TERMINAL_KONSOLE;
     }
     loginfo("got TDA: %s, terminal type %d", str, ictx->initdata->qterm);
   }
@@ -1745,7 +1756,7 @@ build_cflow_automaton(inputctx* ictx){
     { "[?2;0;\\N;\\NS", xtsmgraphics_sixel_cb, },
     { "[>83;\\N;0c", da2_screen_cb, },
     { "[>\\N;\\N;\\Nc", da2_cb, },
-    { "[=\\Sc", da3_cb, }, // CSI da3 form as issued by WezTerm
+    { "[=\\Sc", wezterm_tda_cb, }, // CSI da3 form as issued by WezTerm
     // DCS (\eP...ST)
     { "P0+\\S", NULL, }, // negative XTGETTCAP
     { "P1+r\\S", tcap_cb, }, // positive XTGETTCAP
@@ -1873,6 +1884,7 @@ create_inputctx(tinfo* ti, FILE* infp, int lmargin, int tmargin, int rmargin,
                               i->initdata = NULL;
                               i->coutstanding = 0;
                             }
+                            i->kittykbd = 0;
                             i->iread = i->iwrite = i->ivalid = 0;
                             i->cread = i->cwrite = i->cvalid = 0;
                             i->initdata_complete = NULL;
@@ -2100,7 +2112,7 @@ ictx_independent_p(const inputctx* ictx){
 //  precondition: buflen >= 1. precondition: buf[0] == 0x1b.
 static int
 process_escape(inputctx* ictx, const unsigned char* buf, int buflen){
-  assert(ictx->amata.used < buflen);
+  assert(ictx->amata.used <= buflen);
   while(ictx->amata.used < buflen){
     unsigned char candidate = buf[ictx->amata.used++];
     unsigned used = ictx->amata.used;
@@ -2147,9 +2159,9 @@ process_escape(inputctx* ictx, const unsigned char* buf, int buflen){
 }
 
 // process as many control sequences from |buf|, having |bufused| bytes,
-// as we can. anything not a valid control sequence is dropped. this text
-// needn't be valid UTF-8. this is always called on tbuf; if we find bulk data
-// here, we need replay it into ibuf (assuming that there's room).
+// as we can. this text needn't be valid UTF-8. this is always called on
+// tbuf; if we find bulk data here, we need replay it into ibuf (assuming
+// that there's room).
 static void
 process_escapes(inputctx* ictx, unsigned char* buf, int* bufused){
   int offset = 0;
@@ -2277,6 +2289,7 @@ process_bulk(inputctx* ictx, unsigned char* buf, int* bufused){
 static void
 process_melange(inputctx* ictx, const unsigned char* buf, int* bufused){
   int offset = 0;
+  int origlen = *bufused;
   while(*bufused){
     logdebug("input %d (%u)/%d [0x%02x] (%c)", offset, ictx->amata.used,
              *bufused, buf[offset], isprint(buf[offset]) ? buf[offset] : ' ');
@@ -2285,7 +2298,7 @@ process_melange(inputctx* ictx, const unsigned char* buf, int* bufused){
       consumed = process_escape(ictx, buf + offset, *bufused);
       if(consumed < 0){
         if(ictx->midescape){
-          if(*bufused != -consumed){
+          if(*bufused != -consumed || *bufused == origlen){
             // not at the end; treat it as input. no need to move between
             // buffers; simply ensure we process it as input, and don't mark
             // anything as consumed.
