@@ -50,9 +50,13 @@ get_default_geometry(tinfo* ti){
 // terminal implements DECSDM correctly (enabling it with \e[?80h), or inverts
 // the meaning (*disabling* it with \e[?80h) (we always want it disabled).
 static inline void
-setup_sixel_bitmaps(tinfo* ti, int fd, bool invert80){
-  if(invert80){
-    ti->pixel_init = sixel_init_inverted;
+setup_sixel_bitmaps(tinfo* ti, int fd, unsigned forcesdm, unsigned invert80){
+  if(forcesdm){
+    if(invert80){
+      ti->pixel_init = sixel_init_inverted;
+    }else{
+      ti->pixel_init = sixel_init_forcesdm;
+    }
   }else{
     ti->pixel_init = sixel_init;
   }
@@ -71,6 +75,7 @@ setup_sixel_bitmaps(tinfo* ti, int fd, bool invert80){
   ti->sprixel_scale_height = 6;
   set_pixel_blitter(sixel_blit);
   ti->pixel_implementation = NCPIXEL_SIXEL;
+  ti->pixel_cleanup = sixel_cleanup;
   sprite_init(ti, fd);
 }
 
@@ -157,6 +162,10 @@ query_rgb(void){
 
 void free_terminfo_cache(tinfo* ti){
   stop_inputlayer(ti);
+  loginfo("brought down input layer");
+  if(ti->pixel_cleanup){
+    ti->pixel_cleanup(ti);
+  }
   free(ti->termversion);
   free(ti->esctable);
 #ifdef __linux__
@@ -169,6 +178,7 @@ void free_terminfo_cache(tinfo* ti){
   }
 #endif
   free(ti->tpreserved);
+  loginfo("destroyed terminfo cache");
 }
 
 // compare one terminal version against another. numerics, separated by
@@ -429,35 +439,39 @@ init_terminfo_esc(tinfo* ti, const char* name, escape_e idx,
 #define RMCUP DECRST(SET_SMCUP)
 
 static ssize_t
-send_initial_directives(int fd){
+send_initial_directives(queried_terminals_e qterm, int fd){
+  int total = 0;
 // 4096 is more than sufficient for up through 256 OSC queries
 #define PQUERYBUFLEN 4096
-  char* pqueries = malloc(PQUERYBUFLEN);
-  if(pqueries == NULL){
-    return -1;
-  }
-  int total = 0;
-  // bunch the queries up according to known palette sizes, so that we don't
-  // knock out batched OSCs with error responses.
-  const int qsets[] = { 0, 8, 16, 88, 256 };
-  for(size_t q = 1 ; q < sizeof(qsets) / sizeof(*qsets) ; ++q){
-    int len = 0;
-    for(int i = qsets[q - 1] ; i < qsets[q] ; ++i){
-      len += sprintf(pqueries + len, "\x1b]4;%d;?\e\\", i);
-      assert(len < PQUERYBUFLEN);
-    }
-    if(blocking_write(fd, pqueries, len)){
+  if(qterm != TERMINAL_LINUX){
+    // FIXME linux kernel does not yet support OSC4, and bleeds it. don't send
+    // palette queries on linux VT.
+    char* pqueries = malloc(PQUERYBUFLEN);
+    if(pqueries == NULL){
       return -1;
     }
-    total += len;
+    // bunch the queries up according to known palette sizes, so that we don't
+    // knock out batched OSCs with error responses.
+    const int qsets[] = { 0, 8, 16, 88, 256 };
+    for(size_t q = 1 ; q < sizeof(qsets) / sizeof(*qsets) ; ++q){
+      int len = 0;
+      for(int i = qsets[q - 1] ; i < qsets[q] ; ++i){
+        len += sprintf(pqueries + len, "\x1b]4;%d;?\e\\", i);
+        assert(len < PQUERYBUFLEN);
+      }
+      if(blocking_write(fd, pqueries, len)){
+        return -1;
+      }
+      total += len;
+    }
+    free(pqueries);
   }
-  free(pqueries);
+#undef PQUERYBUFLEN
   if(blocking_write(fd, DIRECTIVES, strlen(DIRECTIVES))){
     return -1;
   }
   total += strlen(DIRECTIVES);
   return total;
-#undef PQUERYBUFLEN
 }
 
 // we send an XTSMGRAPHICS to set up 256 (or ideally 1024) color registers.
@@ -467,9 +481,11 @@ send_initial_directives(int fd){
 // DSRCPR as early as possible, so that it precedes any query material that's
 // bled onto stdin and echoed. if 'noaltscreen' is set, do not send an smcup.
 // if 'draininput' is set, do not send any keyboard modifiers.
+// precondition: ti->ttyfd is a valid fd (we're connected to a terminal)
 static int
-send_initial_queries(int fd, unsigned minimal, unsigned noaltscreen,
+send_initial_queries(tinfo* ti, unsigned minimal, unsigned noaltscreen,
                      unsigned draininput){
+  int fd = ti->ttyfd;
   size_t total = 0;
   // everything sends DSRCPR, and everything sends DIRECTIVES afterwards.
   // we send KKBDENTER immediately before DIRECTIVES unless input is being
@@ -497,7 +513,7 @@ send_initial_queries(int fd, unsigned minimal, unsigned noaltscreen,
     }
     total += strlen(IDQUERIES);
   }
-  ssize_t directiveb = send_initial_directives(fd);
+  ssize_t directiveb = send_initial_directives(ti->qterm, fd);
   if(directiveb < 0){
     return -1;
   }
@@ -715,7 +731,7 @@ apply_kitty_heuristics(tinfo* ti, size_t* tablelen, size_t* tableused){
 
 static const char*
 apply_alacritty_heuristics(tinfo* ti, size_t* tablelen, size_t* tableused,
-                           bool* invertsixel){
+                           bool* forcesdm, bool* invertsixel){
   ti->caps.quadrants = true;
   // ti->caps.sextants = true; // alacritty https://github.com/alacritty/alacritty/issues/4409
   ti->caps.rgb = true;
@@ -723,6 +739,7 @@ apply_alacritty_heuristics(tinfo* ti, size_t* tablelen, size_t* tableused,
   if(add_appsync_escapes_dcs(ti, tablelen, tableused)){
     return NULL;
   }
+  *forcesdm = true;
   if(compare_versions(ti->termversion, "0.15.1") < 0){
     *invertsixel = true;
   }
@@ -741,10 +758,11 @@ apply_vte_heuristics(tinfo* ti, size_t* tablelen, size_t* tableused){
 }
 
 static const char*
-apply_foot_heuristics(tinfo* ti, bool* invertsixel){
+apply_foot_heuristics(tinfo* ti, bool* forcesdm, bool* invertsixel){
   ti->caps.sextants = true;
   ti->caps.quadrants = true;
   ti->caps.rgb = true;
+  *forcesdm = true;
   if(compare_versions(ti->termversion, "1.8.2") < 0){
     *invertsixel = true;
   }
@@ -780,7 +798,8 @@ apply_wezterm_heuristics(tinfo* ti, size_t* tablelen, size_t* tableused){
 
 static const char*
 apply_xterm_heuristics(tinfo* ti, size_t* tablelen, size_t* tableused,
-                       bool* invertsixel){
+                       bool* forcesdm, bool* invertsixel){
+  *forcesdm = true;
   if(compare_versions(ti->termversion, "369") < 0){
     *invertsixel = true; // xterm 369 inverted DECSDM
   }
@@ -795,10 +814,11 @@ apply_xterm_heuristics(tinfo* ti, size_t* tablelen, size_t* tableused,
 
 static const char*
 apply_mintty_heuristics(tinfo* ti, size_t* tablelen, size_t* tableused,
-                        bool* invertsixel){
+                        bool* forcesdm, bool* invertsixel){
   if(add_smulx_escapes(ti, tablelen, tableused)){
     return NULL;
   }
+  *forcesdm = true;
   if(compare_versions(ti->termversion, "3.5.2") < 0){
     *invertsixel = true;
   }
@@ -814,10 +834,15 @@ apply_msterminal_heuristics(tinfo* ti){
 }
 
 static const char*
-apply_contour_heuristics(tinfo* ti, bool* invertsixel){
+apply_contour_heuristics(tinfo* ti, size_t* tablelen, size_t* tableused,
+                         bool* forcesdm, bool* invertsixel){
+  if(add_smulx_escapes(ti, tablelen, tableused)){
+    return NULL;
+  }
   ti->caps.quadrants = true;
   ti->caps.sextants = true;
   ti->caps.rgb = true;
+  *forcesdm = true;
   *invertsixel = true;
   return "Contour";
 }
@@ -842,25 +867,31 @@ apply_rxvt_heuristics(tinfo* ti){
 
 static const char*
 apply_terminology_heuristics(tinfo* ti){
-  ti->caps.rgb = false; // as of at least 1.9.0
+  // no RGB as of at least 1.9.0
   ti->caps.quadrants = true;
   return "Terminology";
 }
 
 static const char*
-apply_linux_heuristics(tinfo* ti, size_t* tablelen, size_t* tableused,
-                       unsigned nonewfonts){
+apply_konsole_heuristics(tinfo* ti){
+  ti->caps.rgb = true;
+  ti->caps.quadrants = true;
+  return "Konsole";
+}
+
+static const char*
+apply_linux_heuristics(tinfo* ti, unsigned nonewfonts){
+  const char* tname = NULL;
 #ifdef __linux__
-  const char* termname = NULL;
   struct utsname un;
   if(uname(&un) == 0){
     ti->termversion = strdup(un.release);
   }
   if(is_linux_framebuffer(ti)){
-    termname = "FBcon";
+    tname = "FBcon";
     setup_fbcon_bitmaps(ti, ti->linux_fb_fd);
   }else{
-    termname = "VT";
+    tname = "VT";
   }
   ti->caps.halfblocks = false;
   ti->caps.braille = false; // no caps.braille, no caps.sextants in linux console
@@ -870,11 +901,10 @@ apply_linux_heuristics(tinfo* ti, size_t* tablelen, size_t* tableused,
   }
   // assume no useful unicode drawing unless we're positively sure
 #else
+  (void)ti;
   (void)nonewfonts;
 #endif
-  (void)tablelen;
-  (void)tableused;
-  return termname;
+  return tname;
 }
 
 // qui si convien lasciare ogne sospetto; ogne viltà convien che qui sia morta.
@@ -886,17 +916,18 @@ apply_linux_heuristics(tinfo* ti, size_t* tablelen, size_t* tableused,
 // though, so it's something of a worst-of-all-worlds deal where TERM still
 // needs be correct, even though we identify the terminal. le sigh.
 static int
-apply_term_heuristics(tinfo* ti, const char* termname, queried_terminals_e qterm,
-                      size_t* tablelen, size_t* tableused, bool* invertsixel,
+apply_term_heuristics(tinfo* ti, const char* tname, queried_terminals_e qterm,
+                      size_t* tablelen, size_t* tableused,
+                      bool* forcesdm, bool* invertsixel,
                       unsigned nonewfonts){
 #ifdef __MINGW32__
   if(qterm == TERMINAL_UNKNOWN){
     qterm = TERMINAL_MSTERMINAL;
   }
 #endif
-  if(!termname){
+  if(!tname){
     // setupterm interprets a missing/empty TERM variable as the special value “unknown”.
-    termname = ti->termname ? ti->termname : "unknown";
+    tname = ti->termname ? ti->termname : "unknown";
   }
   // st had neither caps.sextants nor caps.quadrants last i checked (0.8.4)
   ti->caps.braille = true; // most everyone has working caps.braille, even from fonts
@@ -907,13 +938,14 @@ apply_term_heuristics(tinfo* ti, const char* termname, queried_terminals_e qterm
       newname = apply_kitty_heuristics(ti, tablelen, tableused);
       break;
     case TERMINAL_ALACRITTY:
-      newname = apply_alacritty_heuristics(ti, tablelen, tableused, invertsixel);
+      newname = apply_alacritty_heuristics(ti, tablelen, tableused,
+                                           forcesdm, invertsixel);
       break;
     case TERMINAL_VTE:
       newname = apply_vte_heuristics(ti, tablelen, tableused);
       break;
     case TERMINAL_FOOT:
-      newname = apply_foot_heuristics(ti, invertsixel);
+      newname = apply_foot_heuristics(ti, forcesdm, invertsixel);
       break;
     case TERMINAL_TMUX:
       newname = "tmux"; // FIXME what, oh what to do with tmux?
@@ -928,16 +960,19 @@ apply_term_heuristics(tinfo* ti, const char* termname, queried_terminals_e qterm
       newname = apply_wezterm_heuristics(ti, tablelen, tableused);
       break;
     case TERMINAL_XTERM:
-      newname = apply_xterm_heuristics(ti, tablelen, tableused, invertsixel);
+      newname = apply_xterm_heuristics(ti, tablelen, tableused,
+                                       forcesdm, invertsixel);
       break;
     case TERMINAL_MINTTY:
-      newname = apply_mintty_heuristics(ti, tablelen, tableused, invertsixel);
+      newname = apply_mintty_heuristics(ti, tablelen, tableused,
+                                        forcesdm, invertsixel);
       break;
     case TERMINAL_MSTERMINAL:
       newname = apply_msterminal_heuristics(ti);
       break;
     case TERMINAL_CONTOUR:
-      newname = apply_contour_heuristics(ti, invertsixel);
+      newname = apply_contour_heuristics(ti, tablelen, tableused,
+                                         forcesdm, invertsixel);
       break;
     case TERMINAL_ITERM:
       newname = apply_iterm_heuristics(ti, tablelen, tableused);
@@ -949,20 +984,23 @@ apply_term_heuristics(tinfo* ti, const char* termname, queried_terminals_e qterm
       newname = "Terminal.app"; // no quadrants, no sextants, no rgb, but it does have braille
       break;
     case TERMINAL_LINUX:
-      newname = apply_linux_heuristics(ti, tablelen, tableused, nonewfonts);
+      newname = apply_linux_heuristics(ti, nonewfonts);
       break;
     case TERMINAL_TERMINOLOGY:
       newname = apply_terminology_heuristics(ti);
       break;
+    case TERMINAL_KONSOLE:
+      newname = apply_konsole_heuristics(ti);
+      break;
     default:
-      newname = termname;
+      newname = tname;
       break;
   }
   if(newname == NULL){
     logerror("no name provided for termtype %d", qterm);
     return -1;
   }
-  termname = newname;
+  tname = newname;
   // run a wcwidth(⣿) to guarantee libc Unicode 3 support, independent of term
   if(wcwidth(L'⣿') < 0){
     ti->caps.braille = false;
@@ -971,7 +1009,7 @@ apply_term_heuristics(tinfo* ti, const char* termname, queried_terminals_e qterm
   if(wcwidth(L'🬸') < 0){
     ti->caps.sextants = false;
   }
-  ti->termname = termname;
+  ti->termname = tname;
   return 0;
 }
 
@@ -1182,7 +1220,9 @@ handle_responses(tinfo* ti, size_t* tablelen, size_t* tableused,
   }
   // kitty trumps sixel, when both are available
   if((*kitty_graphics = iresp->kitty_graphics) == 0){
-    ti->color_registers = iresp->color_registers;
+    if((ti->color_registers = iresp->color_registers) > SIXEL_MAX_REGISTERS){
+      ti->color_registers = SIXEL_MAX_REGISTERS;
+    }
     ti->sixel_maxy_pristine = iresp->sixely;
     ti->sixel_maxy = iresp->sixely;
     ti->sixel_maxx = iresp->sixelx;
@@ -1226,6 +1266,7 @@ int interrogate_terminfo(tinfo* ti, FILE* out, unsigned utf8,
     cursor_y = &foolcursor_y;
   }
   *cursor_x = *cursor_y = -1;
+  ti->sixelengine = NULL;
   ti->bg_collides_default = 0xfe000000;
   ti->fg_default = 0xff000000;
   ti->kbdlevel = UINT_MAX; // see comment in tinfo definition
@@ -1276,7 +1317,7 @@ int interrogate_terminfo(tinfo* ti, FILE* out, unsigned utf8,
     // if we already know our terminal (e.g. on the linux console), there's no
     // need to send the identification queries. the controls are sufficient.
     bool minimal = (ti->qterm != TERMINAL_UNKNOWN);
-    if(send_initial_queries(ti->ttyfd, minimal, noaltscreen, draininput)){
+    if(send_initial_queries(ti, minimal, noaltscreen, draininput)){
       goto err;
     }
   }
@@ -1408,16 +1449,17 @@ int interrogate_terminfo(tinfo* ti, FILE* out, unsigned utf8,
       goto err;
     }
   }
+  bool forcesdm = false;
   bool invertsixel = false;
   if(apply_term_heuristics(ti, tname, ti->qterm, &tablelen, &tableused,
-                           &invertsixel, nonewfonts)){
+                           &forcesdm, &invertsixel, nonewfonts)){
     goto err;
   }
   build_supported_styles(ti);
   if(ti->pixel_draw == NULL && ti->pixel_draw_late == NULL){
     // color_registers was only assigned if kitty_graphics were unavailable
     if(ti->color_registers > 0){
-      setup_sixel_bitmaps(ti, ti->ttyfd, invertsixel);
+      setup_sixel_bitmaps(ti, ti->ttyfd, forcesdm, invertsixel);
     }
     if(kitty_graphics){
       setup_kitty_bitmaps(ti, ti->ttyfd, NCPIXEL_KITTY_STATIC);
@@ -1546,18 +1588,18 @@ int cbreak_mode(tinfo* ti){
 }
 
 // replace or populate the TERM environment variable with 'termname'
-int putenv_term(const char* termname){
+int putenv_term(const char* tname){
   #define ENVVAR "TERM"
   const char* oldterm = getenv(ENVVAR);
   if(oldterm){
-    logdebug("replacing %s value %s with %s", ENVVAR, oldterm, termname);
+    logdebug("replacing %s value %s with %s", ENVVAR, oldterm, tname);
   }else{
-    loginfo("provided %s value %s", ENVVAR, termname);
+    loginfo("provided %s value %s", ENVVAR, tname);
   }
-  if(oldterm && strcmp(oldterm, termname) == 0){
+  if(oldterm && strcmp(oldterm, tname) == 0){
     return 0;
   }
-  char* buf = malloc(strlen(termname) + strlen(ENVVAR) + 1);
+  char* buf = malloc(strlen(tname) + strlen(ENVVAR) + 1);
   if(buf == NULL){
     return -1;
   }
