@@ -20,7 +20,7 @@ def channel_rgb8(channel):
 def channel_set_rgb8(channel, r, g, b):
     checkRGB(r, g, b)
     c = (r << 16) | (g << 8) | b
-    return (channel & ~lib.CELL_BG_RGB_MASK) | lib.CELL_BGDEFAULT_MASK | c
+    return (channel & ~lib.NC_BG_RGB_MASK) | lib.NC_BGDEFAULT_MASK | c
 
 def channels_fchannel(channels):
     return channels & 0xffffffff00000000
@@ -103,8 +103,8 @@ class Ncplane:
         return ret
 
     def getDimensions(self):
-        y = ffi.new("int *")
-        x = ffi.new("int *")
+        y = ffi.new("unsigned int *")
+        x = ffi.new("unsigned int *")
         lib.ncplane_dim_yx(self.n, y, x)
         return (y[0], x[0])
 
